@@ -23,17 +23,17 @@
 // one that works, such as, for example, the semigroup in the test
 // KnuthBendixCongruenceByPairs 001.
 
-// There is a fair amount of overlap between CongruenceByPairsHelper and
+// There is a fair amount of overlap between CongruenceByPairs and
 // FroidurePin. In particular, the need to wrangle all of the types, and the
 // InternalProduct, InternalEqualTo, and InternalHash. The main aim of this
 // class is to provide the derived class KnuthBendixByPairs, which is designed
 // to run without fully enumerating the underlying FroidurePin instance. In
 // particular, because this might be infinite. We therefore require all of the
-// machinery to implemented in CongruenceByPairsHelper for KnuthBendixByPairs
+// machinery to implemented in CongruenceByPairs for KnuthBendixByPairs
 // (specifically, we won't know element_index_type's for elements where the
 // underlying FroidurePin<KBE> is infinite) and so we must be able to work
 // directly with the KBE's themselves, rather than with their indices. It would
-// be possible to improve CongruenceByPairsHelper, using:
+// be possible to improve CongruenceByPairs, using:
 //
 // * use pairs of element_index_type and FroidurePin::fast_product
 //   instead of the current setup, i.e. in this case, this would be a
@@ -42,7 +42,7 @@
 //   be passed the FroidurePin as the "state" parameter, and use fast_product
 //   to compute the product.
 //
-// but given that CongruenceByPairsHelper is likely to be a terrible algorithm
+// but given that CongruenceByPairs is likely to be a terrible algorithm
 // even with this optimisation, it does not seem worthwhile implementing the
 // above change.
 //
@@ -70,12 +70,12 @@
 #include "bruidhinn-traits.hpp"  // for detail::BruidhinnTraits
 #include "cong-intf.hpp"         // for CongruenceInterface::class_index_type
 #include "cong-wrap.hpp"         // for CongruenceWrapper
+#include "exception.hpp"         // for LIBSEMIGROUPS_EXCEPTION
 #include "froidure-pin.hpp"      // for FroidurePin
 #include "kbe.hpp"               // for detail::KBE
 #include "knuth-bendix.hpp"      // for fpsemigroup::KnuthBendix
-#include "libsemigroups-exception.hpp"  // for LIBSEMIGROUPS_EXCEPTION
-#include "types.hpp"                    // for word_type
-#include "uf.hpp"                       // for Duf
+#include "types.hpp"             // for word_type
+#include "uf.hpp"                // for Duf
 
 namespace libsemigroups {
   class FroidurePinBase;  // Forward declaration, for constructor parameter
@@ -90,13 +90,9 @@ namespace libsemigroups {
   //! semigroups and monoids.
   //!
   //! This page contains a summary of the member functions of the class
-  //! CongruenceByPairs, and related things in libsemigroups.
-  //! CongruenceByPairsHelper is a helper class, CongruenceByPairs has
-  //! identical functionality, please use CongruenceByPairs, but not
-  //! CongruenceByPairsHelper since this will be deprecated in a later version
-  //! of ``libsemigroups``.
+  //! CongruenceByPairs, and related things in ``libsemigroups``.
   //!
-  //! \sa congruence_type and tril.
+  //! \sa congruence_kind and tril.
   //!
   //! \par Example
   //! \code
@@ -111,31 +107,31 @@ namespace libsemigroups {
   //! using P = CongruenceByPairs<decltype(S)>;
   //!
   //! P cong1(right, S);
-  //! cong1.nr_classes();   // 11804
+  //! cong1.number_of_classes();   // 11804
   //! P cong2(left, S);
-  //! cong2.nr_classes();   // 11804
+  //! cong2.number_of_classes();   // 11804
   //! P cong3(twosided, S);
-  //! cong3.nr_classes();   // 11804
+  //! cong3.number_of_classes();   // 11804
   //! \endcode
 
   // Implemented in cong-pair-impl.hpp
   template <typename TFroidurePinType>
-  class CongruenceByPairsHelper : public CongruenceInterface,
-                                  protected detail::BruidhinnTraits<
-                                      typename TFroidurePinType::element_type> {
+  class CongruenceByPairs : public CongruenceInterface,
+                            protected detail::BruidhinnTraits<
+                                typename TFroidurePinType::element_type> {
     static_assert(
         std::is_base_of<FroidurePinBase, TFroidurePinType>::value,
         "the template parameter must be derived from FroidurePinBase");
 
    public:
     ////////////////////////////////////////////////////////////////////////
-    // CongruenceByPairsHelper - typedefs - public
+    // CongruenceByPairs - typedefs - public
     ////////////////////////////////////////////////////////////////////////
 
     //! No doc
     using froidure_pin_type = TFroidurePinType;
 
-    //! The type of elements over which an instance of CongruenceByPairsHelper
+    //! The type of elements over which an instance of CongruenceByPairs
     //! is defined.
     using element_type = typename froidure_pin_type::element_type;
 
@@ -159,25 +155,25 @@ namespace libsemigroups {
 
    protected:
     ////////////////////////////////////////////////////////////////////////
-    // CongruenceByPairsHelper - typedefs - protected
+    // CongruenceByPairs - typedefs - protected
     ////////////////////////////////////////////////////////////////////////
     //! No doc
     using internal_element_type =
         typename detail::BruidhinnTraits<element_type>::internal_value_type;
 
     ////////////////////////////////////////////////////////////////////////
-    // CongruenceByPairsHelper - constructor - protected
+    // CongruenceByPairs - constructor - protected
     ////////////////////////////////////////////////////////////////////////
     // This is protected because it is not possible to create a
-    // CongruenceByPairsHelper object without it being defined over a parent
+    // CongruenceByPairs object without it being defined over a parent
     // semigroup.
 
     //! No doc
-    explicit CongruenceByPairsHelper(congruence_type);
+    explicit CongruenceByPairs(congruence_kind);
 
    private:
     ////////////////////////////////////////////////////////////////////////
-    // CongruenceByPairsHelper - typedefs - private
+    // CongruenceByPairs - typedefs - private
     ////////////////////////////////////////////////////////////////////////
     using internal_const_element_type = typename detail::BruidhinnTraits<
         element_type>::internal_const_value_type;
@@ -193,19 +189,18 @@ namespace libsemigroups {
 
    public:
     ////////////////////////////////////////////////////////////////////////
-    // CongruenceByPairsHelper - constructor + destructor - public
+    // CongruenceByPairs - constructor + destructor - public
     ////////////////////////////////////////////////////////////////////////
 
     //! No doc
     template <typename T,
-              typename SFINAE
-              = typename std::enable_if<IsState<T>::value, T>::type>
-    CongruenceByPairsHelper(congruence_type type, std::shared_ptr<T> stt, bool)
-        : CongruenceByPairsHelper(type) {
+              typename SFINAE = std::enable_if_t<IsState<T>::value, T>>
+    CongruenceByPairs(congruence_kind type, std::shared_ptr<T> stt, bool)
+        : CongruenceByPairs(type) {
       _state = stt;
     }
 
-    //! Construct a CongruenceByPairsHelper over the FroidurePin instance \p S
+    //! Construct a CongruenceByPairs over the FroidurePin instance \p S
     //! representing a left/right/2-sided congruence according to \p type.
     //! \param type whether the congruence is left, right, or 2-sided
     //! \param S  a shared_ptr to the semigroup over which the congruence is
@@ -213,21 +208,21 @@ namespace libsemigroups {
     //!
     //! \throws LibsemigroupsException if \p type and the template parameter
     //! \p T are incompatible. Currently, this is when \p type is not
-    //! `congruence_type::right` and `T::element_type` is TCE.
+    //! `congruence_kind::right` and `T::element_type` is TCE.
     //!
     //! \par Complexity
     //! Constant.
     //!
     //! \note
     //! The FroidurePinBase pointed to by \p S is not copied
-    CongruenceByPairsHelper(congruence_type                  type,
-                            std::shared_ptr<FroidurePinBase> S) noexcept;
+    CongruenceByPairs(congruence_kind                  type,
+                      std::shared_ptr<FroidurePinBase> S) noexcept;
 
-    //! Construct a CongruenceByPairsHelper over the FroidurePin instance \p fp
+    //! Construct a CongruenceByPairs over the FroidurePin instance \p fp
     //! representing a left/right/2-sided congruence according to \p type.
     //!
     //! \tparam T a class derived from FroidurePinBase. It is required that
-    //! `T::element_type` is the same as CongruenceByPairsHelper::element_type
+    //! `T::element_type` is the same as CongruenceByPairs::element_type
     //! (which is the same as the template parameter \p element_type).
     //!
     //! \param type whether the congruence is left, right, or 2-sided
@@ -236,7 +231,7 @@ namespace libsemigroups {
     //!
     //! \throws LibsemigroupsException if \p type and the template parameter
     //! \p T are incompatible. Currently, this is when \p type is not
-    //! `congruence_type::right` and `T::element_type` is TCE.
+    //! `congruence_kind::right` and `T::element_type` is TCE.
     //!
     //! \par Complexity
     //! Constant.
@@ -244,54 +239,53 @@ namespace libsemigroups {
     //! \warning The parameter `T& S` is copied, this might be expensive, use
     //! a std::shared_ptr to avoid the copy!
     template <typename T>
-    CongruenceByPairsHelper(congruence_type type, T const& S)
-        : CongruenceByPairsHelper(type,
-                                  static_cast<std::shared_ptr<FroidurePinBase>>(
-                                      std::make_shared<T>(S))) {
+    CongruenceByPairs(congruence_kind type, T const& S)
+        : CongruenceByPairs(type,
+                            static_cast<std::shared_ptr<FroidurePinBase>>(
+                                std::make_shared<T>(S))) {
       static_assert(std::is_base_of<FroidurePinBase, T>::value,
                     "the template parameter must be a derived class of "
                     "FroidurePinBase");
       static_assert(
           std::is_same<typename T::element_type, element_type>::value,
           "incompatible element_type's, the element_type of the FroidurePin "
-          "instance must be the same as the CongruenceByPairsHelper "
+          "instance must be the same as the CongruenceByPairs "
           "element_type");
-      if (type != congruence_type::right
+      if (type != congruence_kind::right
           && std::is_same<typename T::element_type, detail::TCE>::value) {
         LIBSEMIGROUPS_EXCEPTION("Cannot create a left or 2-sided congruence "
                                 "over a semigroup of TCE's");
       }
     }
 
-    ~CongruenceByPairsHelper();
+    ~CongruenceByPairs();
 
-    //! A CongruenceByPairsHelper instance is not default-constructible.
+    //! A CongruenceByPairs instance is not default-constructible.
     //! This constructor is deleted.
-    CongruenceByPairsHelper() = delete;
+    CongruenceByPairs() = delete;
 
     //! A default copy constructor.
-    CongruenceByPairsHelper(CongruenceByPairsHelper const&) = default;
+    CongruenceByPairs(CongruenceByPairs const&) = default;
 
-    //! A CongruenceByPairsHelper instance is not copy assignable.
+    //! A CongruenceByPairs instance is not copy assignable.
     //! This constructor is deleted.
-    CongruenceByPairsHelper& operator=(CongruenceByPairsHelper const&) = delete;
+    CongruenceByPairs& operator=(CongruenceByPairs const&) = delete;
 
     //! A default move constructor.
-    CongruenceByPairsHelper(CongruenceByPairsHelper&&) = default;
+    CongruenceByPairs(CongruenceByPairs&&) = default;
 
-    //! A CongruenceByPairsHelper instance is not move assignable.
+    //! A CongruenceByPairs instance is not move assignable.
     //! This constructor is deleted.
-    CongruenceByPairsHelper& operator=(CongruenceByPairsHelper&&) = delete;
+    CongruenceByPairs& operator=(CongruenceByPairs&&) = delete;
 
    protected:
     ////////////////////////////////////////////////////////////////////////
-    // CongruenceByPairsHelper - member functions - protected
+    // CongruenceByPairs - member functions - protected
     ////////////////////////////////////////////////////////////////////////
 
     //! No doc
     template <typename SFINAE = state_type>
-    auto state() ->
-        typename std::enable_if<IsState<SFINAE>::value, SFINAE&>::type {
+    auto state() -> std::enable_if_t<IsState<SFINAE>::value, SFINAE&> {
       return *_state;
     }
 
@@ -304,7 +298,7 @@ namespace libsemigroups {
 
     // non_trivial_classes_impl is private in CongruenceInterface but
     // protected here since KnuthBendixCongruenceByPairs uses it and
-    // derives from class CongruenceByPairsHelper.
+    // derives from class CongruenceByPairs.
     //! No doc
     std::shared_ptr<non_trivial_classes_type const>
     non_trivial_classes_impl() override;
@@ -325,7 +319,7 @@ namespace libsemigroups {
     ////////////////////////////////////////////////////////////////////////
 
     word_type class_index_to_word_impl(class_index_type) override;
-    size_t    nr_classes_impl() override;
+    size_t    number_of_classes_impl() override;
     std::shared_ptr<FroidurePinBase> quotient_impl() override;
     class_index_type word_to_class_index_impl(word_type const&) override;
 
@@ -340,7 +334,7 @@ namespace libsemigroups {
     }
 
     ////////////////////////////////////////////////////////////////////////
-    // CongruenceByPairsHelper - member functions - private
+    // CongruenceByPairs - member functions - private
     ////////////////////////////////////////////////////////////////////////
 
     size_t add_index(internal_element_type) const;
@@ -349,7 +343,7 @@ namespace libsemigroups {
     void   init();
 
     ////////////////////////////////////////////////////////////////////////
-    // CongruenceByPairsHelper - inner structs - private
+    // CongruenceByPairs - inner structs - private
     ////////////////////////////////////////////////////////////////////////
 
     struct InternalEqualTo : private detail::BruidhinnTraits<element_type> {
@@ -372,9 +366,8 @@ namespace libsemigroups {
                       const_reference x,
                       const_reference y,
                       void*,
-                      size_t tid = 0) ->
-          typename std::enable_if<std::is_void<state_type>::value,
-                                  SFINAE>::type {
+                      size_t tid = 0)
+          -> std::enable_if_t<std::is_void<state_type>::value, SFINAE> {
         Product()(xy, x, y, tid);
       }
 
@@ -383,9 +376,8 @@ namespace libsemigroups {
                       const_reference x,
                       const_reference y,
                       state_type*     stt,
-                      size_t          tid = 0) ->
-          typename std::enable_if<!std::is_void<state_type>::value,
-                                  SFINAE>::type {
+                      size_t          tid = 0)
+          -> std::enable_if_t<!std::is_void<state_type>::value, SFINAE> {
         Product()(xy, x, y, stt, tid);
       }
     };
@@ -410,7 +402,7 @@ namespace libsemigroups {
     };
 
     ////////////////////////////////////////////////////////////////////////
-    // CongruenceByPairsHelper - data - private
+    // CongruenceByPairs - data - private
     ////////////////////////////////////////////////////////////////////////
 
     mutable std::vector<class_index_type> _class_lookup;
@@ -437,30 +429,6 @@ namespace libsemigroups {
     internal_element_type                      _tmp2;
   };
 
-  // The following only exist for backwards compatibility, so that it is
-  // possible to create a CongruenceByPairs where the template parameter is the
-  // element type. TODO(2.0): remove these and rename CongruenceByPairsHelper
-  // -> CongruenceByPairs.
-  template <typename T, typename = void>
-  class CongruenceByPairs;
-
-  template <typename T>
-  class CongruenceByPairs<
-      T,
-      typename std::enable_if<std::is_base_of<FroidurePinBase, T>::value>::type>
-      : public CongruenceByPairsHelper<T> {
-    using CongruenceByPairsHelper<T>::CongruenceByPairsHelper;
-  };
-
-  template <typename T>
-  class CongruenceByPairs<
-      T,
-      typename std::enable_if<
-          !std::is_base_of<FroidurePinBase, T>::value>::type>
-      : public CongruenceByPairsHelper<FroidurePin<T>> {
-    using CongruenceByPairsHelper<FroidurePin<T>>::CongruenceByPairsHelper;
-  };
-
   //! Defined in ``cong-pair.hpp``.
   //!
   //! On this page, we describe the functionality relating to the brute force
@@ -478,7 +446,7 @@ namespace libsemigroups {
   //! to enumerate the infinite semigroup represented by the output of
   //! fpsemigroup::KnuthBendix first).
   //!
-  //! \sa congruence_type, tril, and CongruenceByPairs.
+  //! \sa congruence_kind, tril, and CongruenceByPairs.
   //!
   //! \par Example
   //! \code
@@ -497,20 +465,20 @@ namespace libsemigroups {
   //! KnuthBendixCongruenceByPairs kbp(twosided, kb);
   //! kbp.add_pair({0}, {1});
   //!
-  //! kbp.nr_non_trivial_classes();  // == 1
+  //! kbp.number_of_non_trivial_classes();  // == 1
   //! kbp.cbegin_ntc()->size();      // == 5
   //! \endcode
   // Implemented in cong-pair.cpp.
   class KnuthBendixCongruenceByPairs
-      : public CongruenceByPairsHelper<FroidurePin<
+      : public CongruenceByPairs<FroidurePin<
             detail::KBE,
             FroidurePinTraits<detail::KBE, fpsemigroup::KnuthBendix>>> {
     ////////////////////////////////////////////////////////////////////////
     // KnuthBendixCongruenceByPairs - typedefs - private
     ////////////////////////////////////////////////////////////////////////
 
-    using KnuthBendix              = fpsemigroup::KnuthBendix;
-    using CongruenceByPairsHelper_ = CongruenceByPairsHelper<
+    using KnuthBendix        = fpsemigroup::KnuthBendix;
+    using CongruenceByPairs_ = CongruenceByPairs<
         FroidurePin<detail::KBE,
                     FroidurePinTraits<detail::KBE, fpsemigroup::KnuthBendix>>>;
 
@@ -534,7 +502,7 @@ namespace libsemigroups {
     //!
     //! \note
     //! The fpsemigroup::KnuthBendix referred to by \p kb is not copied.
-    KnuthBendixCongruenceByPairs(congruence_type              type,
+    KnuthBendixCongruenceByPairs(congruence_kind              type,
                                  std::shared_ptr<KnuthBendix> kb) noexcept;
 
     //! Construct a KnuthBendixCongruenceByPairs over the
@@ -551,7 +519,7 @@ namespace libsemigroups {
     //!
     //! \warning The parameter \p kb is copied, this might be expensive, use a
     //! std::shared_ptr to avoid the copy!
-    KnuthBendixCongruenceByPairs(congruence_type    type,
+    KnuthBendixCongruenceByPairs(congruence_kind    type,
                                  KnuthBendix const& kb) noexcept;
 
     //! Deleted.
@@ -575,14 +543,16 @@ namespace libsemigroups {
     ////////////////////////////////////////////////////////////////////////
     // Runner - pure virtual member functions - public
     ////////////////////////////////////////////////////////////////////////
-    void run_impl() override;
-    bool finished_impl() const override;
+
+    void                             run_impl() override;
+    bool                             finished_impl() const override;
+    std::shared_ptr<FroidurePinBase> quotient_impl() override;
 
     ////////////////////////////////////////////////////////////////////////
     // CongruenceInterface - non-pure virtual member functions - private
     ////////////////////////////////////////////////////////////////////////
 
-    using CongruenceByPairsHelper_::non_trivial_classes_impl;
+    using CongruenceByPairs_::non_trivial_classes_impl;
     // Override the method for the class CongruenceByPairs to avoid having to
     // know the parent semigroup (found as part of
     // KnuthBendixCongruenceByPairs::run) to add a pair.
