@@ -18,16 +18,25 @@
 #ifndef LIBSEMIGROUPS_DIGRAPH_HELPER_HPP_
 #define LIBSEMIGROUPS_DIGRAPH_HELPER_HPP_
 
-#include <cstddef>  // for size_t
-#include <stack>    // for stack
-#include <utility>  // for pair
-#include <vector>   // for vector
+#include <algorithm>         // for fill
+#include <cstddef>           // for size_t
+#include <cstdint>           // for uint8_t
+#include <initializer_list>  // for initializer_list
+#include <stack>             // for stack
+#include <string>            // for to_string, string
+#include <tuple>             // for tie
+#include <utility>           // for pair
+#include <vector>            // for vector
 
-#include "constants.hpp"                // for UNDEFINED
-#include "libsemigroups-exception.hpp"  // for LIBSEMIGROUPS_EXCEPTION
-#include "types.hpp"                    // for word_type
+#include "constants.hpp"  // for UNDEFINED
+#include "debug.hpp"      // for LIBSEMIGROUPS_ASSERT
+#include "exception.hpp"  // for LIBSEMIGROUPS_EXCEPTION
+#include "types.hpp"      // for word_type
+#include "uf.hpp"         // for Duf
 
 namespace libsemigroups {
+  struct ActionDigraphBase;  // forward decl
+
   template <typename T>
   class ActionDigraph;  // forward decl
 
@@ -43,11 +52,11 @@ namespace libsemigroups {
     //! No doc
     // not noexcept because it throws an exception!
     template <typename T>
-    void validate_node(ActionDigraph<T> const& ad, node_type<T> const v) {
-      if (v >= ad.nr_nodes()) {
+    void validate_node(ActionDigraph<T> const& ad, node_type<T> v) {
+      if (v >= ad.number_of_nodes()) {
         LIBSEMIGROUPS_EXCEPTION("node value out of bounds, expected value in "
                                 "the range [0, %d), got %d",
-                                ad.nr_nodes(),
+                                ad.number_of_nodes(),
                                 v);
       }
     }
@@ -55,7 +64,7 @@ namespace libsemigroups {
     //! No doc
     // not noexcept because it throws an exception!
     template <typename T>
-    void validate_label(ActionDigraph<T> const& ad, label_type<T> const lbl) {
+    void validate_label(ActionDigraph<T> const& ad, label_type<T> lbl) {
       if (lbl >= ad.out_degree()) {
         LIBSEMIGROUPS_EXCEPTION("label value out of bounds, expected value in "
                                 "the range [0, %d), got %d",
@@ -74,7 +83,7 @@ namespace libsemigroups {
     //!
     //! \returns
     //! A value of type ActionDigraph::node_type. If one or more edges in
-    //! \p path are not defined, then libsemigroups::UNDEFINED is returned.
+    //! \p path are not defined, then \ref UNDEFINED is returned.
     //!
     //! \throw LibsemigroupsException if \p first is not a node in the digraph
     //! or \p path contains a value that is not an edge-label.
@@ -96,10 +105,154 @@ namespace libsemigroups {
       return last;
     }
 
+    // TODO(later) follow_path non-nc version for iterators
+
+    //! Follow the path from a specified node labelled by a word.
+    //!
+    //! This function returns the last node on the path in the action digraph
+    //! \p ad starting at the node \p from labelled by \f$[first, last)\f$ or
+    //! libsemigroups::UNDEFINED.
+    //!
+    //! \param ad an action digraph
+    //! \param from the source node
+    //! \param first iterator into a word
+    //! \param last iterator into a word.
+    //!
+    //! \exception
+    //! \noexcept
+    //!
+    //! \returns A value of type ActionDigraph::node_type.
+    //!
+    //! \complexity
+    //! At worst the distance from \p first to \p last.
+    //!
+    //! \warning
+    //! No checks on the arguments of this function are performed.
+    template <typename T, typename S>
+    node_type<T> follow_path_nc(ActionDigraph<T> const& ad,
+                                node_type<T> const      from,
+                                S                       first,
+                                S                       last) noexcept {
+      node_type<T> to = from;
+      for (auto it = first; it < last && to != UNDEFINED; ++it) {
+        to = ad.unsafe_neighbor(to, *it);
+      }
+      return to;
+    }
+
+    //! Follow the path from a specified node labelled by a word.
+    //!
+    //! This function returns the last node on the path in the action digraph
+    //! \p ad starting at the node \p from labelled by \p path or
+    //! libsemigroups::UNDEFINED.
+    //!
+    //! \tparam T the node type of the action digraph
+    //!
+    //! \param ad an action digraph
+    //! \param from the source node
+    //! \param path the word
+    //!
+    //! \exception
+    //! \noexcept
+    //!
+    //! \returns A value of type ActionDigraph::node_type.
+    //!
+    //! \complexity
+    //! At worst the length of \p path.
+    //!
+    //! \warning
+    //! No checks on the arguments of this function are performed.
+    template <typename T>
+    node_type<T> follow_path_nc(ActionDigraph<T> const& ad,
+                                node_type<T> const      from,
+                                word_type const&        path) noexcept {
+      return follow_path_nc(ad, from, path.cbegin(), path.cend());
+    }
+
+    //! Returns the last node on the path labelled by a word and an iterator to
+    //! the position in the word reached.
+    //!
+    //! \tparam T the node type of the action digraph
+    //! \tparam S the type of the iterators into a word
+    //!
+    //! \param ad an action digraph
+    //! \param from the source node
+    //! \param first iterator into a word
+    //! \param last iterator into a word.
+    //!
+    //! \exception
+    //! \noexcept
+    //!
+    //! \returns A pair consisting of ActionDigraph::node_type and \p S.
+    //!
+    //! \complexity
+    //! At worst the distance from \p first to \p last.
+    //!
+    //! \warning
+    //! No checks on the arguments of this function are performed.
+    template <typename T, typename S>
+    std::pair<node_type<T>, S> last_node_on_path_nc(ActionDigraph<T> const& ad,
+                                                    node_type<T> from,
+                                                    S const&     first,
+                                                    S const& last) noexcept {
+      auto         it   = first;
+      node_type<T> prev = from;
+      node_type<T> to   = from;
+      for (; it < last && to != UNDEFINED; ++it) {
+        prev = to;
+        to   = ad.unsafe_neighbor(to, *it);
+      }
+      if (it != last || to == UNDEFINED) {
+        LIBSEMIGROUPS_ASSERT(prev != UNDEFINED);
+        return {prev, it - 1};
+      } else {
+        return {to, it};
+      }
+    }
+
+    //! Returns the last node on the path labelled by a word and an iterator to
+    //! the position in the word reached.
+    //!
+    //! \tparam T the node type of the action digraph
+    //! \tparam S the type of the iterators into a word
+    //!
+    //! \param ad an action digraph
+    //! \param from the source node
+    //! \param first iterator into a word
+    //! \param last iterator into a word.
+    //!
+    //! \throws LibsemigroupsException if any of the letters in word `[first,
+    //! last)` is out of bounds.
+    //!
+    //! \returns A pair consisting of ActionDigraph::node_type and \p S.
+    //!
+    //! \complexity
+    //! At worst the distance from \p first to \p last.
+    template <typename T, typename S>
+    std::pair<node_type<T>, S> last_node_on_path(ActionDigraph<T> const& ad,
+                                                 node_type<T>            from,
+                                                 S const&                first,
+                                                 S const&                last) {
+      auto         it   = first;
+      node_type<T> prev = from;
+      node_type<T> to   = from;
+      for (; it < last && to != UNDEFINED; ++it) {
+        prev = to;
+        to   = ad.neighbor(to, *it);
+      }
+      if (it != last || to == UNDEFINED) {
+        LIBSEMIGROUPS_ASSERT(prev != UNDEFINED);
+        return {prev, it - 1};
+      } else {
+        return {to, it};
+      }
+    }
+
     namespace detail {
       template <typename T>
       using stack_type  = std::stack<std::pair<node_type<T>, label_type<T>>>;
       using lookup_type = std::vector<uint8_t>;
+
       template <typename T>
       using topological_sort_type = std::vector<node_type<T>>;
 
@@ -111,38 +264,38 @@ namespace libsemigroups {
       // topological_sort.
       template <typename T>
       bool is_acyclic(ActionDigraph<T> const& ad,
-                      stack_type<T>&          stck,
-                      lookup_type&            seen) {
+                      std::stack<T>&          stck,
+                      std::vector<T>&         preorder,
+                      T&                      next_preorder_num,
+                      std::vector<T>&         postorder,
+                      T&                      next_postorder_num) {
         size_t const M = ad.out_degree();
-        do {
-          auto& p = stck.top();
-          if (seen[p.first] == 2) {
-            return false;
-          } else if (seen[p.first] == 1 || p.second >= M) {
-            // backtrack
-            seen[p.first] = 1;
-            stck.pop();
-            seen[stck.top().first] = 3;
-            stck.top().second++;
+        size_t const N = ad.number_of_nodes();
+        node_type<T> v;
+        while (!stck.empty()) {
+          v = stck.top();
+          stck.pop();
+          if (v >= N) {
+            postorder[v - N] = next_postorder_num++;
           } else {
-            node_type<T> node;
-            std::tie(node, p.second)
-                = ad.unsafe_next_neighbor(p.first, p.second);
-#ifdef LIBSEMIGROUPS_DEBUG
-            if (node == UNDEFINED) {
-              LIBSEMIGROUPS_ASSERT(p.second == UNDEFINED);
-            } else {
-              validate_node(ad, node);
-              validate_label(ad, p.second);
-            }
-#endif
-            if (node != UNDEFINED) {
-              seen[p.first] = 2;
-              stck.emplace(node, 0);
+            if (preorder[v] < next_preorder_num && postorder[v] == N) {
+              // v is an ancestor of some vertex later in the search
+              return false;
+            } else if (preorder[v] == N) {
+              // not seen v before
+              preorder[v] = next_preorder_num++;
+              // acts as a divider, so that we know when we've stopped
+              // processing the out-neighbours of v
+              stck.push(N + v);
+              for (size_t label = 0; label < M; ++label) {
+                auto w = ad.unsafe_neighbor(v, label);
+                if (w != UNDEFINED) {
+                  stck.push(w);
+                }
+              }
             }
           }
-        } while (stck.size() > 1);
-        seen[stck.top().first] = 1;
+        }
         return true;
       }
 
@@ -199,7 +352,8 @@ namespace libsemigroups {
 
     //! Check if a digraph is acyclic.
     //!
-    //! \tparam T the type used as the template parameter for the ActionDigraph.
+    //! \tparam T the type used as the template parameter for the
+    //! ActionDigraph.
     //!
     //! \param ad the ActionDigraph object to check.
     //!
@@ -210,10 +364,10 @@ namespace libsemigroups {
     //! \no_libsemigroups_except
     //!
     //! \par Complexity
-    //! \f$O(m + n)\f$ where \f$m\f$ is the number of nodes in the ActionDigraph
-    //! \p ad and \f$n\f$ is the number of edges. Note that for ActionDigraph
-    //! objects the number of edges is always at most \f$mk\f$ where \f$k\f$ is
-    //! the ActionDigraph::out_degree.
+    //! \f$O(m + n)\f$ where \f$m\f$ is the number of nodes in the
+    //! ActionDigraph \p ad and \f$n\f$ is the number of edges. Note that for
+    //! ActionDigraph objects the number of edges is always at most \f$mk\f$
+    //! where \f$k\f$ is the ActionDigraph::out_degree.
     //!
     //! A digraph is acyclic if every directed cycle on the digraph is
     //! trivial.
@@ -233,18 +387,24 @@ namespace libsemigroups {
       if (ad.validate()) {
         return false;
       }
-      node_type<T> const                                 N = ad.nr_nodes();
-      std::stack<std::pair<node_type<T>, label_type<T>>> stck;
-      std::vector<uint8_t>                               seen(N, 0);
+      auto const     N = ad.number_of_nodes();
+      std::stack<T>  stck;
+      std::vector<T> preorder(N, N);
+      T              next_preorder_num = 0;
+      std::vector<T> postorder(N, N);
+      T              next_postorder_num = 0;
 
       for (node_type<T> m = 0; m < N; ++m) {
-        if (seen[m] == 0) {
-          stck.emplace(m, 0);
-          if (!detail::is_acyclic(ad, stck, seen)) {
-            LIBSEMIGROUPS_ASSERT(seen[m] != 0);
+        if (preorder[m] == N) {
+          stck.push(m);
+          if (!detail::is_acyclic(ad,
+                                  stck,
+                                  preorder,
+                                  next_preorder_num,
+                                  postorder,
+                                  next_postorder_num)) {
             return false;
           }
-          LIBSEMIGROUPS_ASSERT(seen[m] != 0);
         }
       }
       return true;
@@ -254,10 +414,11 @@ namespace libsemigroups {
     //! possible.
     //!
     //! If it is not empty, the returned vector has the property that if an
-    //! edge from a node \c n points to a node \c m, then \c m occurs before \c
-    //! n in the vector.
+    //! edge from a node \c n points to a node \c m, then \c m occurs before
+    //! \c n in the vector.
     //!
-    //! \tparam T the type used as the template parameter for the ActionDigraph.
+    //! \tparam T the type used as the template parameter for the
+    //! ActionDigraph.
     //!
     //! \param ad the ActionDigraph object to check.
     //!
@@ -269,10 +430,10 @@ namespace libsemigroups {
     //! \no_libsemigroups_except
     //!
     //! \par Complexity
-    //! \f$O(m + n)\f$ where \f$m\f$ is the number of nodes in the ActionDigraph
-    //! \p ad and \f$n\f$ is the number of edges. Note that for ActionDigraph
-    //! objects the number of edges is always at most \f$mk\f$ where \f$k\f$ is
-    //! the ActionDigraph::out_degree.
+    //! \f$O(m + n)\f$ where \f$m\f$ is the number of nodes in the
+    //! ActionDigraph \p ad and \f$n\f$ is the number of edges. Note that for
+    //! ActionDigraph objects the number of edges is always at most \f$mk\f$
+    //! where \f$k\f$ is the ActionDigraph::out_degree.
     template <typename T>
     detail::topological_sort_type<T>
     topological_sort(ActionDigraph<T> const& ad) {
@@ -283,7 +444,7 @@ namespace libsemigroups {
         return order;
       }
 
-      size_t const          N = ad.nr_nodes();
+      size_t const          N = ad.number_of_nodes();
       detail::stack_type<T> stck;
       std::vector<uint8_t>  seen(N, 0);
 
@@ -298,7 +459,7 @@ namespace libsemigroups {
           }
         }
       }
-      LIBSEMIGROUPS_ASSERT(order.size() == ad.nr_nodes());
+      LIBSEMIGROUPS_ASSERT(order.size() == ad.number_of_nodes());
       return order;
     }
 
@@ -310,9 +471,11 @@ namespace libsemigroups {
     //! before \c n in the vector, and the last item in the vector is \p
     //! source.
     //!
-    //! \tparam T the type used as the template parameter for the ActionDigraph.
+    //! \tparam T the type used as the template parameter for the
+    //! ActionDigraph.
     //!
     //! \param ad the ActionDigraph object to check.
+    //! \param source the source node.
     //!
     //! \returns
     //! A std::vector<ActionDigraph<T>::node_type> that contains the nodes of
@@ -327,12 +490,12 @@ namespace libsemigroups {
     //! and \f$n\f$ is the number of edges.
     template <typename T>
     detail::topological_sort_type<T>
-    topological_sort(ActionDigraph<T> const& ad, node_type<T> const source) {
+    topological_sort(ActionDigraph<T> const& ad, node_type<T> source) {
       detail::topological_sort_type<T> order;
       if (ad.validate()) {
         return order;
       }
-      size_t const          N = ad.nr_nodes();
+      size_t const          N = ad.number_of_nodes();
       detail::stack_type<T> stck;
       std::vector<uint8_t>  seen(N, 0);
 
@@ -344,7 +507,8 @@ namespace libsemigroups {
     //! Check if the subdigraph induced by the nodes reachable from a source
     //! node is acyclic.
     //!
-    //! \tparam T the type used as the template parameter for the ActionDigraph.
+    //! \tparam T the type used as the template parameter for the
+    //! ActionDigraph.
     //!
     //! \param ad the ActionDigraph object to check.
     //! \param source the source node.
@@ -356,10 +520,10 @@ namespace libsemigroups {
     //! \no_libsemigroups_except
     //!
     //! \par Complexity
-    //! \f$O(m + n)\f$ where \f$m\f$ is the number of nodes in the ActionDigraph
-    //! \p ad and \f$n\f$ is the number of edges. Note that for ActionDigraph
-    //! objects the number of edges is always at most \f$mk\f$ where \f$k\f$ is
-    //! the ActionDigraph::out_degree.
+    //! \f$O(m + n)\f$ where \f$m\f$ is the number of nodes in the
+    //! ActionDigraph \p ad and \f$n\f$ is the number of edges. Note that for
+    //! ActionDigraph objects the number of edges is always at most \f$mk\f$
+    //! where \f$k\f$ is the ActionDigraph::out_degree.
     //!
     //! A digraph is acyclic if every directed cycle on the digraph is
     //! trivial.
@@ -380,17 +544,23 @@ namespace libsemigroups {
     //! \endcode
     // Not noexcept because detail::is_acyclic isn't
     template <typename T>
-    bool is_acyclic(ActionDigraph<T> const& ad, node_type<T> const source) {
+    bool is_acyclic(ActionDigraph<T> const& ad, node_type<T> source) {
       validate_node(ad, source);
-      std::stack<std::pair<node_type<T>, label_type<T>>> stck;
-      stck.emplace(source, 0);
-      std::vector<uint8_t> seen(ad.nr_nodes(), 0);
-      return detail::is_acyclic(ad, stck, seen);
+      auto const    N = ad.number_of_nodes();
+      std::stack<T> stck;
+      stck.push(source);
+      std::vector<T> preorder(N, N);
+      T              next_preorder_num = 0;
+      std::vector<T> postorder(N, N);
+      T              next_postorder_num = 0;
+      return detail::is_acyclic(
+          ad, stck, preorder, next_preorder_num, postorder, next_postorder_num);
     }
 
     //! Check if there is a path from one node to another.
     //!
-    //! \tparam T the type used as the template parameter for the ActionDigraph.
+    //! \tparam T the type used as the template parameter for the
+    //! ActionDigraph.
     //!
     //! \param ad the ActionDigraph object to check.
     //! \param source the source node.
@@ -403,10 +573,10 @@ namespace libsemigroups {
     //! \no_libsemigroups_except
     //!
     //! \par Complexity
-    //! \f$O(m + n)\f$ where \f$m\f$ is the number of nodes in the ActionDigraph
-    //! \p ad and \f$n\f$ is the number of edges. Note that for ActionDigraph
-    //! objects the number of edges is always at most \f$mk\f$ where \f$k\f$ is
-    //! the ActionDigraph::out_degree.
+    //! \f$O(m + n)\f$ where \f$m\f$ is the number of nodes in the
+    //! ActionDigraph \p ad and \f$n\f$ is the number of edges. Note that for
+    //! ActionDigraph objects the number of edges is always at most \f$mk\f$
+    //! where \f$k\f$ is the ActionDigraph::out_degree.
     //!
     //! \note
     //! If \p source and \p target are equal, then, by convention, we consider
@@ -438,7 +608,7 @@ namespace libsemigroups {
       label_type<T>             edge = 0;
       std::stack<node_type<T>>  nodes;
       std::stack<label_type<T>> edges;
-      std::vector<bool>         seen(ad.nr_nodes(), false);
+      std::vector<bool>         seen(ad.number_of_nodes(), false);
       nodes.push(source);
       seen[source] = true;
 
@@ -469,10 +639,37 @@ namespace libsemigroups {
       return false;
     }
 
+    template <typename T>
+    bool is_acyclic(ActionDigraph<T> const& ad,
+                    node_type<T>            source,
+                    node_type<T>            target) {
+      validate_node(ad, source);
+      validate_node(ad, target);
+      if (!is_reachable(ad, source, target)) {
+        return true;
+      }
+      auto const    N = ad.number_of_nodes();
+      std::stack<T> stck;
+      stck.push(source);
+      std::vector<T> preorder(N, N);
+      T              next_preorder_num = 0;
+      std::vector<T> postorder(N, N);
+      T              next_postorder_num = 0;
+      // TODO(later) there should be a better way of doing this
+      for (auto it = ad.cbegin_nodes(); it != ad.cend_nodes(); ++it) {
+        if (!is_reachable(ad, *it, target)) {
+          preorder[*it] = N + 1;
+        }
+      }
+      return detail::is_acyclic(
+          ad, stck, preorder, next_preorder_num, postorder, next_postorder_num);
+    }
+
     //! Adds a cycle involving the specified range of nodes.
     //!
-    //! \tparam T the type used as the template parameter for the ActionDigraph.
-    //! \tparam U the type of an iterator pointing to nodes of an ActionDigraph
+    //! \tparam T the type used as the template parameter for the
+    //! ActionDigraph. \tparam U the type of an iterator pointing to nodes of
+    //! an ActionDigraph
     //!
     //! \param ad the ActionDigraph object to add a cycle to.
     //! \param first a const iterator to nodes of \p ad
@@ -490,7 +687,7 @@ namespace libsemigroups {
     //! \note
     //! The edges added by this function are all labelled \c 0.
     template <typename T, typename U>
-    void add_cycle(ActionDigraph<T>& ad, U const first, U const last) {
+    void add_cycle(ActionDigraph<T>& ad, U first, U last) {
       for (auto it = first; it < last - 1; ++it) {
         ad.add_edge(*it, *(it + 1), 0);
       }
@@ -499,7 +696,8 @@ namespace libsemigroups {
 
     //! Adds a cycle consisting of \p N new nodes.
     //!
-    //! \tparam T the type used as the template parameter for the ActionDigraph.
+    //! \tparam T the type used as the template parameter for the
+    //! ActionDigraph.
     //!
     //! \param ad the ActionDigraph object to add a cycle to.
     //! \param N the length of the cycle and number of new nodes to add.
@@ -516,8 +714,8 @@ namespace libsemigroups {
     //! \note
     //! The edges added by this function are all labelled \c 0.
     template <typename T>
-    void add_cycle(ActionDigraph<T>& ad, size_t const N) {
-      size_t M = ad.nr_nodes();
+    void add_cycle(ActionDigraph<T>& ad, size_t N) {
+      size_t M = ad.number_of_nodes();
       ad.add_nodes(N);
       add_cycle(ad, ad.cbegin_nodes() + M, ad.cend_nodes());
     }
@@ -526,7 +724,7 @@ namespace libsemigroups {
       template <typename T>
       std::string to_string(ActionDigraph<T>& ad) {
         std::string out = "ActionDigraph<size_t> ad;\n";
-        out += "ad.add_nodes(" + std::to_string(ad.nr_nodes()) + ");\n";
+        out += "ad.add_nodes(" + std::to_string(ad.number_of_nodes()) + ");\n";
         out += "ad.add_to_out_degree(" + std::to_string(ad.out_degree())
                + ");\n";
         for (auto n = ad.cbegin_nodes(); n < ad.cend_nodes(); ++n) {
@@ -541,6 +739,164 @@ namespace libsemigroups {
         return out;
       }
     }  // namespace detail
-  }    // namespace action_digraph_helper
+
+    //! Constructs a digraph from number of nodes and an \c initializer_list.
+    //!
+    //! This function constructs an ActionDigraph from its arguments whose
+    //! out-degree is specified by the length of the first \c initializer_list
+    //! in the 2nd parameter.
+    //!
+    //! \tparam T the type of the nodes of the digraph
+    //!
+    //! \param num_nodes the number of nodes in the digraph.
+    //! \param il the out-neighbors of the digraph.
+    //!
+    //! \returns A value of type ActionDigraph.
+    //!
+    //! \throws LibsemigroupsException
+    //! if ActionDigraph<T>::add_edge throws when adding edges from \p il.
+    //!
+    //! \complexity
+    //! \f$O(mn)\f$ where \f$m\f$ is the length of \p il and \f$n\f$ is the
+    //! parameter \p num_nodes.
+    //!
+    //! \par Example
+    //! \code
+    //! // Construct an action digraph with 5 nodes and 10 edges (7 specified)
+    //! action_digraph_helper::make<uint8_t>(
+    //!     5, {{0, 0}, {1, 1}, {2}, {3, 3}});
+    //! \endcode
+    template <typename T>
+    ActionDigraph<T> make(size_t num_nodes,
+                          std::initializer_list<std::initializer_list<T>> il) {
+      ActionDigraph<T> result(num_nodes, il.begin()->size());
+      for (size_t i = 0; i < il.size(); ++i) {
+        for (size_t j = 0; j < (il.begin() + i)->size(); ++j) {
+          auto val = *((il.begin() + i)->begin() + j);
+          if (val != UNDEFINED) {
+            result.add_edge(i, *((il.begin() + i)->begin() + j), j);
+          }
+        }
+      }
+      return result;
+    }
+
+    //! Check if a digraph is connected.
+    //!
+    //! \tparam T the type used as the template parameter for the
+    //! ActionDigraph.
+    //!
+    //! \param ad the ActionDigraph object to check.
+    //!
+    //! \returns
+    //! A value of type `bool`.
+    //!
+    //! \exceptions
+    //! \no_libsemigroups_except
+    //!
+    //! \par Complexity
+    //! \f$O(m + n)\f$ where \f$m\f$ is the number of nodes in the
+    //! ActionDigraph \p ad and \f$n\f$ is the number of edges. Note that for
+    //! ActionDigraph objects the number of edges is always at most \f$mk\f$
+    //! where \f$k\f$ is the ActionDigraph::out_degree.
+    //!
+    //! A digraph is *connected* if for every pair of nodes \f$u\f$ and
+    //! \f$v\f$ there exists a sequence \f$u_0 := u, \ldots, u_{n - 1} := v\f$
+    //! such that either  \f$(u_i, u_{i + 1})\f$ or \f$(u_{i + 1}, u_i)\f$ is
+    //! an edge. Note that \f$u\f$ and \f$v\f$ can be equal, and the sequence
+    //! above can be of length \f$0\f$.
+    //!
+    //! \par Example
+    //! \code
+    //! auto ad = action_digraph_helper::make<uint8_t>(
+    //!     5, {{0, 0}, {1, 1}, {2}, {3, 3}});
+    //! action_digraph_helper::is_connected(ad);  // returns false
+    //! \endcode
+    template <typename T>
+    bool is_connected(ActionDigraph<T> const& ad) {
+      using node_type = typename ActionDigraph<T>::node_type;
+
+      auto const N = ad.number_of_nodes();
+      if (N == 0) {
+        return true;
+      }
+
+      ::libsemigroups::detail::Duf<> uf(N);
+      for (node_type n = 0; n < N; ++n) {
+        for (auto it = ad.cbegin_edges(n); it != ad.cend_edges(n); ++it) {
+          if (*it != UNDEFINED) {
+            uf.unite(n, *it);
+          }
+        }
+      }
+      return uf.number_of_blocks() == 1;
+    }
+
+    //! Check if a digraph is strictly cyclic.
+    //!
+    //! \tparam T the type used as the template parameter for the
+    //! ActionDigraph.
+    //!
+    //! \param ad the ActionDigraph object to check.
+    //!
+    //! \returns
+    //! A value of type `bool`.
+    //!
+    //! \exceptions
+    //! \no_libsemigroups_except
+    //!
+    //! \par Complexity
+    //! \f$O(m + n)\f$ where \f$m\f$ is the number of nodes in the
+    //! ActionDigraph \p ad and \f$n\f$ is the number of edges. Note that for
+    //! ActionDigraph objects the number of edges is always at most \f$mk\f$
+    //! where \f$k\f$ is the ActionDigraph::out_degree.
+    //!
+    //! A digraph is *strictly cyclic* if there exists a node \f$v\f$ from
+    //! which every node is reachable (including \f$v\f$). There must be a
+    //! path of length at least \f$1\f$ from the original node \f$v\f$ to
+    //! itself (i.e. \f$v\f$ is not considered to be reachable from itself by
+    //! default).
+    //!
+    //! \par Example
+    //! \code
+    //! auto ad = action_digraph_helper::make<uint8_t>(
+    //!     5, {{0, 0}, {1, 1}, {2}, {3, 3}});
+    //! action_digraph_helper::is_strictly_cyclic(ad);  // returns false
+    //! \endcode
+    template <typename T>
+    bool is_strictly_cyclic(ActionDigraph<T> const& ad) {
+      using node_type = typename ActionDigraph<T>::node_type;
+      auto const N    = ad.number_of_nodes();
+
+      if (N == 0) {
+        return true;
+      }
+
+      std::vector<bool> seen(N, false);
+      std::stack<T>     stack;
+
+      for (node_type m = 0; m < N; ++m) {
+        stack.push(m);
+        size_t count = 0;
+        while (!stack.empty()) {
+          auto n = stack.top();
+          stack.pop();
+          if (!seen[n]) {
+            seen[n] = true;
+            if (++count == N) {
+              return true;
+            }
+            for (auto it = ad.cbegin_edges(n); it != ad.cend_edges(n); ++it) {
+              if (*it != UNDEFINED) {
+                stack.push(*it);
+              }
+            }
+          }
+        }
+        std::fill(seen.begin(), seen.end(), false);
+      }
+      return false;
+    }
+  }  // namespace action_digraph_helper
 }  // namespace libsemigroups
 #endif  // LIBSEMIGROUPS_DIGRAPH_HELPER_HPP_
