@@ -82,12 +82,10 @@ namespace libsemigroups {
 
     // Helpers
     template <typename T, typename S>
-    using ReturnTypeIfExists =
-        typename std::enable_if<!std::is_void<T>::value, S>::type;
+    using ReturnTypeIfExists = std::enable_if_t<!std::is_void<T>::value, S>;
 
     template <typename T, typename S>
-    using ReturnTypeIfNotExists =
-        typename std::enable_if<std::is_void<T>::value, S>::type;
+    using ReturnTypeIfNotExists = std::enable_if_t<std::is_void<T>::value, S>;
 
     struct Iterator {};
     struct ConstIterator : Iterator {};
@@ -123,10 +121,10 @@ namespace libsemigroups {
           std::is_nothrow_copy_constructible<internal_iterator_type>::value)
           : _wrapped_iter(it) {}
 
-      IteratorBase(IteratorBase const&) = default;
+      IteratorBase(IteratorBase const&)            = default;
       IteratorBase& operator=(IteratorBase const&) = default;
 
-      IteratorBase(IteratorBase&&) = default;
+      IteratorBase(IteratorBase&&)            = default;
       IteratorBase& operator=(IteratorBase&&) = default;
 
       virtual ~IteratorBase() = default;
@@ -175,6 +173,7 @@ namespace libsemigroups {
                 typename TOperator = typename TSfinae::LessOrEqualTo>
       auto constexpr operator<=(IteratorBase const& that) const noexcept
           -> ReturnTypeIfNotExists<TOperator, bool> {
+        // Shouldn't this just use < or ==?
         return _wrapped_iter <= that._wrapped_iter;
       }
 
@@ -303,55 +302,67 @@ namespace libsemigroups {
                 typename TOperator = typename TSfinae::EqualTo>
       auto constexpr operator==(IteratorStatefulBase const& that) const noexcept
           -> ReturnTypeIfExists<TOperator, bool> {
-        return TOperator()(
-            get_state(), this->get_wrapped_iter(), that.get_wrapped_iter());
+        return TOperator()(this->get_state(),
+                           this->get_wrapped_iter(),
+                           that.get_state(),
+                           that.get_wrapped_iter());
       }
 
       template <typename TSfinae   = TIteratorTraits,
                 typename TOperator = typename TSfinae::NotEqualTo>
       auto constexpr operator!=(IteratorStatefulBase const& that) const noexcept
           -> ReturnTypeIfExists<TOperator, bool> {
-        return TOperator()(
-            get_state(), this->get_wrapped_iter(), that.get_wrapped_iter());
+        return TOperator()(this->get_state(),
+                           this->get_wrapped_iter(),
+                           that.get_state(),
+                           that.get_wrapped_iter());
       }
 
       template <typename TSfinae   = TIteratorTraits,
                 typename TOperator = typename TSfinae::Less>
       auto constexpr operator<(IteratorStatefulBase const& that) const noexcept
           -> ReturnTypeIfExists<TOperator, bool> {
-        return TOperator()(
-            get_state(), this->get_wrapped_iter(), that.get_wrapped_iter());
+        return TOperator()(this->get_state(),
+                           this->get_wrapped_iter(),
+                           that.get_state(),
+                           that.get_wrapped_iter());
       }
 
       template <typename TSfinae   = TIteratorTraits,
                 typename TOperator = typename TSfinae::More>
       auto constexpr operator>(IteratorStatefulBase const& that) const noexcept
           -> ReturnTypeIfExists<TOperator, bool> {
-        return TOperator()(
-            get_state(), this->get_wrapped_iter(), that.get_wrapped_iter());
+        return TOperator()(this->get_state(),
+                           this->get_wrapped_iter(),
+                           that.get_state(),
+                           that.get_wrapped_iter());
       }
 
       template <typename TSfinae   = TIteratorTraits,
                 typename TOperator = typename TSfinae::LessOrEqualTo>
       auto constexpr operator<=(IteratorStatefulBase const& that) const noexcept
           -> ReturnTypeIfExists<TOperator, bool> {
-        return TOperator()(
-            get_state(), this->get_wrapped_iter(), that.get_wrapped_iter());
+        return TOperator()(this->get_state(),
+                           this->get_wrapped_iter(),
+                           that.get_state(),
+                           that.get_wrapped_iter());
       }
 
       template <typename TSfinae   = TIteratorTraits,
                 typename TOperator = typename TSfinae::MoreOrEqualTo>
       auto constexpr operator>=(IteratorStatefulBase const& that) const noexcept
           -> ReturnTypeIfExists<TOperator, bool> {
-        return TOperator()(
-            get_state(), this->get_wrapped_iter(), that.get_wrapped_iter());
+        return TOperator()(this->get_state(),
+                           this->get_wrapped_iter(),
+                           that.get_state(),
+                           that.get_wrapped_iter());
       }
 
       template <typename TSfinae   = TIteratorTraits,
                 typename TOperator = typename TSfinae::AddAssign>
       auto operator+=(size_type val) noexcept
           -> ReturnTypeIfExists<TOperator, TSubclass&> {
-        TOperator()(get_state(), this->get_wrapped_iter(), val);
+        TOperator()(this->get_state(), this->get_wrapped_iter(), val);
         return static_cast<TSubclass&>(*this);
       }
 
@@ -359,7 +370,7 @@ namespace libsemigroups {
                 typename TOperator = typename TSfinae::SubtractAssign>
       auto operator-=(size_type val) noexcept
           -> ReturnTypeIfExists<TOperator, TSubclass&> {
-        TOperator()(get_state(), this->get_wrapped_iter(), val);
+        TOperator()(this->get_state(), this->get_wrapped_iter(), val);
         return static_cast<TSubclass&>(*this);
       }
 
@@ -367,14 +378,14 @@ namespace libsemigroups {
       template <typename TSfinae   = TIteratorTraits,
                 typename TOperator = typename TSfinae::PrefixIncrement>
       auto operator++() noexcept -> ReturnTypeIfExists<TOperator, TSubclass&> {
-        TOperator()(get_state(), this->get_wrapped_iter());
+        TOperator()(this->get_state(), this->get_wrapped_iter());
         return static_cast<TSubclass&>(*this);
       }
 
       template <typename TSfinae   = TIteratorTraits,
                 typename TOperator = typename TSfinae::PrefixDecrement>
       auto operator--() noexcept -> ReturnTypeIfExists<TOperator, TSubclass&> {
-        TOperator()(get_state(), this->get_wrapped_iter());
+        TOperator()(this->get_state(), this->get_wrapped_iter());
         return static_cast<TSubclass&>(*this);
       }
 
@@ -382,8 +393,10 @@ namespace libsemigroups {
                 typename TOperator = typename TSfinae::Difference>
       auto constexpr operator-(IteratorStatefulBase const& that) const noexcept
           -> ReturnTypeIfExists<TOperator, difference_type> {
-        return TOperator()(
-            get_state(), this->get_wrapped_iter(), that.get_wrapped_iter());
+        return TOperator()(this->get_state(),
+                           this->get_wrapped_iter(),
+                           that.get_state(),
+                           that.get_wrapped_iter());
       }
 
       template <typename TSfinae   = TIteratorTraits,
@@ -452,10 +465,10 @@ namespace libsemigroups {
           std::is_nothrow_copy_constructible<IteratorBaseAlias>::value)
           : IteratorBaseAlias(it) {}
 
-      IteratorStatelessBase(IteratorStatelessBase const&) = default;
+      IteratorStatelessBase(IteratorStatelessBase const&)            = default;
       IteratorStatelessBase& operator=(IteratorStatelessBase const&) = default;
 
-      IteratorStatelessBase(IteratorStatelessBase&&) = default;
+      IteratorStatelessBase(IteratorStatelessBase&&)            = default;
       IteratorStatelessBase& operator=(IteratorStatelessBase&&) = default;
 
       virtual ~IteratorStatelessBase() = default;
@@ -470,10 +483,10 @@ namespace libsemigroups {
       template <typename TSfinae   = TIteratorTraits,
                 typename TOperator = typename TSfinae::EqualTo>
       auto constexpr
-      operator==(IteratorStatelessBase const& that) const noexcept ->
-          typename std::enable_if<
+      operator==(IteratorStatelessBase const& that) const noexcept
+          -> std::enable_if_t<
               std::is_trivially_default_constructible<TOperator>::value,
-              bool>::type {
+              bool> {
         return TOperator()(this->get_wrapped_iter(), that.get_wrapped_iter());
       }
 
@@ -572,7 +585,7 @@ namespace libsemigroups {
     };
 
     template <typename TIteratorTraits>
-    class ConstIteratorStateful final
+    class ConstIteratorStateful
         : public ConstIterator,
           public IteratorStatefulBase<ConstIteratorStateful<TIteratorTraits>,
                                       TIteratorTraits> {
@@ -601,10 +614,10 @@ namespace libsemigroups {
           std::is_nothrow_copy_constructible<IteratorStatefulBaseAlias>::value)
           : IteratorStatefulBaseAlias(stt, it) {}
 
-      ConstIteratorStateful(ConstIteratorStateful const&) = default;
+      ConstIteratorStateful(ConstIteratorStateful const&)            = default;
       ConstIteratorStateful& operator=(ConstIteratorStateful const&) = default;
 
-      ConstIteratorStateful(ConstIteratorStateful&&) = default;
+      ConstIteratorStateful(ConstIteratorStateful&&)            = default;
       ConstIteratorStateful& operator=(ConstIteratorStateful&&) = default;
 
       ~ConstIteratorStateful() = default;
@@ -689,7 +702,7 @@ namespace libsemigroups {
       ConstIteratorStateless& operator=(ConstIteratorStateless const&)
           = default;
 
-      ConstIteratorStateless(ConstIteratorStateless&&) = default;
+      ConstIteratorStateless(ConstIteratorStateless&&)            = default;
       ConstIteratorStateless& operator=(ConstIteratorStateless&&) = default;
 
       ~ConstIteratorStateless() = default;
@@ -772,10 +785,10 @@ namespace libsemigroups {
                                    IteratorStatefulBaseAlias>::value)
           : IteratorStatefulBaseAlias(stt, it) {}
 
-      IteratorStateful(IteratorStateful const&) = default;
+      IteratorStateful(IteratorStateful const&)            = default;
       IteratorStateful& operator=(IteratorStateful const&) = default;
 
-      IteratorStateful(IteratorStateful&&) = default;
+      IteratorStateful(IteratorStateful&&)            = default;
       IteratorStateful& operator=(IteratorStateful&&) = default;
 
       ~IteratorStateful() = default;
@@ -864,10 +877,10 @@ namespace libsemigroups {
           std::is_nothrow_copy_constructible<IteratorStatelessBaseAlias>::value)
           : IteratorStatelessBaseAlias(it) {}
 
-      IteratorStateless(IteratorStateless const&) = default;
+      IteratorStateless(IteratorStateless const&)            = default;
       IteratorStateless& operator=(IteratorStateless const&) = default;
 
-      IteratorStateless(IteratorStateless&&) = default;
+      IteratorStateless(IteratorStateless&&)            = default;
       IteratorStateless& operator=(IteratorStateless&&) = default;
 
       ~IteratorStateless() = default;
@@ -928,8 +941,8 @@ namespace libsemigroups {
     };
 
     template <typename TIteratorType,
-              typename = typename std::enable_if<
-                  std::is_base_of<Iterator, TIteratorType>::value>::type>
+              typename = std::enable_if_t<
+                  std::is_base_of<Iterator, TIteratorType>::value>>
     TIteratorType operator+(TIteratorType const& cpy, size_t val) {
       TIteratorType out(cpy);
       out += val;
@@ -937,8 +950,8 @@ namespace libsemigroups {
     }
 
     template <typename TIteratorType,
-              typename = typename std::enable_if<
-                  std::is_base_of<Iterator, TIteratorType>::value>::type>
+              typename = std::enable_if_t<
+                  std::is_base_of<Iterator, TIteratorType>::value>>
     TIteratorType operator-(TIteratorType const& cpy, size_t val) {
       TIteratorType out(cpy);
       out -= val;
@@ -946,15 +959,15 @@ namespace libsemigroups {
     }
 
     template <typename TIteratorType,
-              typename = typename std::enable_if<
-                  std::is_base_of<Iterator, TIteratorType>::value>::type>
+              typename = std::enable_if_t<
+                  std::is_base_of<Iterator, TIteratorType>::value>>
     TIteratorType operator+(size_t val, TIteratorType it) {
       return it + val;
     }
 
     template <typename TIteratorType,
-              typename = typename std::enable_if<
-                  std::is_base_of<Iterator, TIteratorType>::value>::type>
+              typename = std::enable_if_t<
+                  std::is_base_of<Iterator, TIteratorType>::value>>
     TIteratorType operator-(size_t val, TIteratorType it) {
       return it - val;
     }
