@@ -34,6 +34,8 @@
 #include "digraph.hpp"       // for ActionDigraph
 #include "fpsemi-intf.hpp"   // for FpSemigroupInterface
 #include "froidure-pin.hpp"  // for FroidurePin
+#include "make-present.hpp"  // for Presentation
+#include "present.hpp"       // for Presentation
 #include "types.hpp"         // for word_type
 #include "word.hpp"          // for word_to_string
 
@@ -52,13 +54,13 @@ namespace libsemigroups {
     //! On this page we describe the functionality relating to the Knuth-Bendix
     //! algorithm for semigroups and monoids that is available in
     //! ``libsemigroups``. This page contains a details of the member functions
-    //! of the class libsemigroups::fpsemigroup::KnuthBendix.
+    //! of the class fpsemigroup::KnuthBendix.
     //!
     //! This class is used to represent a
     //! [string rewriting system](https://w.wiki/9Re)
     //! defining a finitely presented monoid or semigroup.
     //!
-    //! \sa libsemigroups::congruence::KnuthBendix.
+    //! \sa congruence::KnuthBendix.
     //!
     //! \par Example
     //! \code
@@ -73,7 +75,7 @@ namespace libsemigroups {
     //!
     //! !kb.confluent();       // true
     //! kb.run();
-    //! kb.nr_active_rules();  // 31
+    //! kb.number_of_active_rules();  // 31
     //! kb.confluent();        // true
     //! \endcode
     class KnuthBendix final : public FpSemigroupInterface {
@@ -85,14 +87,16 @@ namespace libsemigroups {
       // KnuthBendix - types - public
       //////////////////////////////////////////////////////////////////////////
 
-      //! This type contains various enums for specifying policies to a
+      //! This type contains various enums for specifying certain options to a
       //! KnuthBendix instance.
-      struct policy {
+      struct options {
+        //! Values for specifying how to measure the length of an overlap.
+        //!
         //! The values in this enum determine how a KnuthBendix instance
         //! measures the length \f$d(AB, BC)\f$ of the overlap of two words
         //! \f$AB\f$ and \f$BC\f$:
         //!
-        //! \sa KnuthBendix::overlap_policy(policy::overlap)
+        //! \sa overlap_policy(options::overlap)
         enum class overlap {
           //! \f$d(AB, BC) = |A| + |B| + |C|\f$
           ABC = 0,
@@ -105,8 +109,8 @@ namespace libsemigroups {
 
       //! The type of the return value of froidure_pin().
       //!
-      //! froidure_pin() returns a ``std::shared_ptr`` to a FroidurePinBase,
-      //! which is really of type froidure_pin_type.
+      //! froidure_pin() returns a \shared_ptr to a FroidurePinBase,
+      //! which is really of type \ref froidure_pin_type.
       using froidure_pin_type
           = FroidurePin<detail::KBE,
                         FroidurePinTraits<detail::KBE, KnuthBendix>>;
@@ -115,17 +119,19 @@ namespace libsemigroups {
       // KnuthBendix - constructors and destructor - public
       //////////////////////////////////////////////////////////////////////////
 
+      //! Default constructor.
+      //!
       //! Constructs a KnuthBendix instance with no rules, and the short-lex
       //! reduction ordering.
       //!
-      //! \par Parameters
+      //! \parameters
       //! (None)
       //!
       //! \complexity
       //! Constant.
       KnuthBendix();
 
-      //! Constructs a KnuthBendix instance from a FroidurePin instance.
+      //! Constructs from a FroidurePin instance.
       //!
       //! \param S the FroidurePin instance.
       //!
@@ -136,11 +142,17 @@ namespace libsemigroups {
         init_from(S);
       }
 
-      //! \copydoc KnuthBendix(FroidurePinBase&).
+      //! Constructs from a shared pointer to a FroidurePin instance.
+      //!
+      //! \param S the FroidurePin instance.
+      //!
+      //! \complexity
+      //! \f$O(|S||A|)\f$ where \f$A\f$ is the set of generators used to define
+      //! \p S.
       explicit KnuthBendix(std::shared_ptr<FroidurePinBase> S)
           : KnuthBendix(*S) {}
 
-      //! Copy construct a KnuthBendix instance.
+      //! Copy constructor.
       //!
       //! \param copy the KnuthBendix instance to copy.
       //!
@@ -150,7 +162,7 @@ namespace libsemigroups {
       KnuthBendix(KnuthBendix const& copy);
 
       //! Deleted.
-      KnuthBendix(KnuthBendix&&) = delete;
+      KnuthBendix(KnuthBendix&&);
 
       //! Deleted.
       KnuthBendix& operator=(KnuthBendix const&) = delete;
@@ -166,15 +178,15 @@ namespace libsemigroups {
 
       //! Set the interval at which confluence is checked.
       //!
-      //! KnuthBendix::knuth_bendix periodically checks if
+      //! The function \ref run periodically checks if
       //! the system is already confluent. This function can be used to
       //! set how frequently this happens, it is the number of new overlaps
       //! that should be considered before checking confluence. Setting this
       //! value too low can adversely affect the performance of
-      //! KnuthBendix::knuth_bendix.
+      //! \ref run.
       //!
-      //! The default value is 4096, and should be set to
-      //! libsemigroups::LIMIT_MAX if KnuthBendix::knuth_bendix should never
+      //! The default value is \c 4096, and should be set to
+      //! \ref LIMIT_MAX if \ref run should never
       //! check if the system is already confluent.
       //!
       //! \param val the new value of the interval.
@@ -185,7 +197,7 @@ namespace libsemigroups {
       //! \complexity
       //! Constant.
       //!
-      //! \sa KnuthBendix::knuth_bendix.
+      //! \sa \ref run.
       KnuthBendix& check_confluence_interval(size_t val) {
         _settings._check_confluence_interval = val;
         return *this;
@@ -195,10 +207,10 @@ namespace libsemigroups {
       //!
       //! This function can be used to specify the maximum length of the
       //! overlap of two left hand sides of rules that should be considered in
-      //! KnuthBendix::knuth_bendix.
+      //! \ref run.
       //!
       //! If this value is less than the longest left hand side of a rule, then
-      //! KnuthBendix::knuth_bendix can terminate without the system being
+      //! \ref run can terminate without the system being
       //! confluent.
       //!
       //! \param val the new value of the maximum overlap length.
@@ -209,7 +221,7 @@ namespace libsemigroups {
       //! \complexity
       //! Constant.
       //!
-      //! \sa KnuthBendix::knuth_bendix.
+      //! \sa \ref run.
       KnuthBendix& max_overlap(size_t val) {
         _settings._max_overlap = val;
         return *this;
@@ -219,11 +231,11 @@ namespace libsemigroups {
       //!
       //! This member function sets the (approximate) maximum number of rules
       //! that the system should contain. If this is number is exceeded in
-      //! calls to KnuthBendix::knuth_bendix or
-      //! KnuthBendix::knuth_bendix_by_overlap_length, then they
+      //! calls to \ref run or
+      //! knuth_bendix_by_overlap_length, then they
       //! will terminate and the system may not be confluent.
       //!
-      //! By default this value is libsemigroups::POSITIVE_INFINITY.
+      //! By default this value is \ref POSITIVE_INFINITY.
       //!
       //! \param val the maximum number of rules.
       //!
@@ -233,7 +245,7 @@ namespace libsemigroups {
       //! \complexity
       //! Constant.
       //!
-      //! \sa KnuthBendix::knuth_bendix and KnuthBendix::knuth_bendix.
+      //! \sa \ref run.
       KnuthBendix& max_rules(size_t val) {
         _settings._max_rules = val;
         return *this;
@@ -252,8 +264,8 @@ namespace libsemigroups {
       //! \complexity
       //! Constant.
       //!
-      //! \sa KnuthBendix::policy::overlap.
-      KnuthBendix& overlap_policy(policy::overlap val);
+      //! \sa options::overlap.
+      KnuthBendix& overlap_policy(options::overlap val);
 
       //////////////////////////////////////////////////////////////////////////
       // KnuthBendix - member functions for rules and rewriting - public
@@ -271,11 +283,11 @@ namespace libsemigroups {
       //! \complexity
       //! Constant.
       //!
-      //! \par Parameters
+      //! \parameters
       //! (None)
-      size_t nr_active_rules() const noexcept;
+      size_t number_of_active_rules() const noexcept;
 
-      //! Returns a copy of the active rules of the KnuthBendix instance.
+      //! Returns a copy of the active rules.
       //!
       //! This member function returns a vector consisting of the pairs of
       //! strings which represent the rules of the KnuthBendix instance. The \c
@@ -292,7 +304,7 @@ namespace libsemigroups {
       //! \f$O(n)\f$ where \f$n\f$ is the sum of the lengths of the words in
       //! rules of \p copy.
       //!
-      //! \par Parameters
+      //! \parameters
       //! (None)
       // TODO(later) delete this
       std::vector<rule_type> active_rules() const;
@@ -359,6 +371,7 @@ namespace libsemigroups {
         using PostfixIncrement = void;
       };
 
+      //! Type of an const iterator to a normal form.
       using const_normal_form_iterator
           = detail::ConstIteratorStateful<NormalFormsIteratorTraits>;
 
@@ -373,8 +386,8 @@ namespace libsemigroups {
       static_assert(std::is_destructible<const_normal_form_iterator>::value,
                     "forward iterator requires destructible");
 
-      //! Returns a forward iterator pointing at the first normal form whose
-      //! length is in the given range using the specified alphabet.
+      //! Returns a forward iterator pointing at the first normal form with
+      //! length in a given range.
       //!
       //! If incremented, the iterator will point to the next least short-lex
       //! normal form (if it's less than \p max in length).  Iterators of the
@@ -386,30 +399,29 @@ namespace libsemigroups {
       //! \param max one larger than the maximum length of a normal form.
       //!
       //! \returns
-      //! A value of type `const_normal_form_iterator`.
+      //! A value of type \ref const_normal_form_iterator.
       //!
       //! \exceptions
       //! \no_libsemigroups_except
       //!
       //! \warning
-      //! Copying iterators of this type is expensive.  As a consequence, prefix
-      //! incrementing \c ++it the iterator \c it returned by \c
-      //! cbegin_normal_forms is significantly cheaper than postfix
+      //! Copying iterators of this type is relatively expensive.  As a
+      //! consequence, prefix incrementing \c ++it the iterator \c it returned
+      //! by \c cbegin_normal_forms is significantly cheaper than postfix
       //! incrementing \c it++.
       //!
       //! \warning
-      //! If the finitely presented semigroup represented by \c this is infinite
-      //! then \p max should be chosen with some care.
+      //! If the finitely presented semigroup represented by \c this is
+      //! infinite, then \p max should be chosen with some care.
       //!
       //! \sa
-      //! KnuthBendix::cend_normal_forms.
+      //! \ref cend_normal_forms.
       const_normal_form_iterator cbegin_normal_forms(std::string const& lphbt,
-                                                     size_t const       min,
-                                                     size_t const       max);
+                                                     size_t             min,
+                                                     size_t             max);
 
-      //! Returns a forward iterator pointing at the first normal form whose
-      //! length is in the given range using the alphabet returned by
-      //! KnuthBendix::alphabet.
+      //! Returns a forward iterator pointing at the first normal form with
+      //! length in a given range.
       //!
       //! If incremented, the iterator will point to the next least short-lex
       //! normal form (if it's less than \p max in length).  Iterators of the
@@ -436,23 +448,22 @@ namespace libsemigroups {
       //! then \p max should be chosen with some care.
       //!
       //! \sa
-      //! KnuthBendix::cend_normal_forms.
-      const_normal_form_iterator cbegin_normal_forms(size_t const min,
-                                                     size_t const max) {
+      //! cend_normal_forms.
+      const_normal_form_iterator cbegin_normal_forms(size_t min, size_t max) {
         return cbegin_normal_forms(alphabet(), min, max);
       }
 
       //! Returns a forward iterator pointing to one after the last normal form.
       //!
       //! The iterator returned by this function can be compared with the
-      //! return value of KnuthBendix::cbegin_normal_forms with any parameters.
+      //! return value of \ref cbegin_normal_forms with any parameters.
       //!
       //! \warning The iterator returned by this function may still
-      //! dereferencable and incrementable, but will not point to a normal_form
+      //! dereferenceable and incrementable, but will not point to a normal form
       //! in the correct range.
       //!
       //! \sa
-      //! KnuthBendix::cbegin_normal_forms.
+      //! \ref cbegin_normal_forms.
       const_normal_form_iterator cend_normal_forms() {
         using state_type = NormalFormsIteratorTraits::state_type;
         return const_normal_form_iterator(state_type("", ""),
@@ -494,23 +505,21 @@ namespace libsemigroups {
       // KnuthBendix - main member functions - public
       //////////////////////////////////////////////////////////////////////////
 
-      //! Check if the KnuthBendix instance is confluent.
+      //! Check confluence of the current rules.
       //!
       //! \returns \c true if the KnuthBendix instance is
       //! [confluent](https://w.wiki/9DA) and \c false if it is not.
       //!
-      //! \par Parameters
+      //! \parameters
       //! (None)
       bool confluent() const;
 
-      //! Run the [Knuth-Bendix algorithm](https://w.wiki/9Cz)
-      //! on the KnuthBendix instance.
+      //! Run the Knuth-Bendix by considering all overlaps of a given length.
       //!
-      //! This function runs the Knuth-Bendix algorithm on the rewriting
-      //! system represented by a KnuthBendix instance by considering all
-      //! overlaps of a given length \f$n\f$ (according to the
-      //! KnuthBendix::overlap_measure) before those overlaps of length
-      //! \f$n + 1\f$.
+      //! This function runs the Knuth-Bendix algorithm on the rewriting system
+      //! represented by a KnuthBendix instance by considering all overlaps of
+      //! a given length \f$n\f$ (according to the \ref options::overlap) before
+      //! those overlaps of length \f$n + 1\f$.
       //!
       //! \returns
       //! (None)
@@ -521,15 +530,15 @@ namespace libsemigroups {
       //! \warning This will terminate when the KnuthBendix instance is
       //! confluent, which might be never.
       //!
-      //! \sa KnuthBendix::run.
+      //! \sa \ref run.
       //!
-      //! \par Parameters
+      //! \parameters
       //! (None)
       void knuth_bendix_by_overlap_length();
 
-      //! Returns the Gilman digraph (or automata) of \c this.
+      //! Returns the Gilman digraph.
       //!
-      //! \returns A const reference to a ActionDigraph<size_t>.
+      //! \returns A const reference to a \ref ActionDigraph.
       //!
       //! \exceptions
       //! \no_libsemigroups_except
@@ -537,15 +546,14 @@ namespace libsemigroups {
       //! \warning This will terminate when the KnuthBendix instance is
       //! reduced and confluent, which might be never.
       //!
-      //! \sa KnuthBendix::number_of_normal_forms,
-      //! KnuthBendix::cbegin_normal_forms, and KnuthBendix::cend_normal_forms.
+      //! \sa \ref number_of_normal_forms,
+      //! \ref cbegin_normal_forms, and \ref cend_normal_forms.
       //!
-      //! \par Parameters
+      //! \parameters
       //! (None)
       ActionDigraph<size_t> const& gilman_digraph();
 
-      //! Returns whether or not the empty string belongs to the finitely
-      //! presented semigroup represented by \c this.
+      //! Returns whether or not the empty string belongs in the system.
       //!
       //! \returns
       //! A value of type `bool`.
@@ -556,7 +564,7 @@ namespace libsemigroups {
       //! \complexity
       //! \f$O(n)\f$ where \f$n\f$ is the number of rules.
       //!
-      //! \par Parameters
+      //! \parameters
       //! (None)
       bool contains_empty_string() const;
 
@@ -576,8 +584,8 @@ namespace libsemigroups {
       //! Assuming that \c this has been run until finished, the complexity of
       //! this function is at worst \f$O(mn)\f$ where \f$m\f$ is the number of
       //! letters in the alphabet, and \f$n\f$ is the number of nodes in the
-      //! KnuthBendix::gilman_digraph.
-      uint64_t number_of_normal_forms(size_t const min, size_t const max);
+      //! \ref gilman_digraph.
+      uint64_t number_of_normal_forms(size_t min, size_t max);
 
       //////////////////////////////////////////////////////////////////////////
       // FpSemigroupInterface - pure virtual member functions - public
@@ -590,7 +598,7 @@ namespace libsemigroups {
       //! is infinite. Moreover, the complexity of this function is at worst
       //! \f$O(mn)\f$ where \f$m\f$ is the number of letters in the alphabet,
       //! and \f$n\f$ is the number of nodes in the
-      //! KnuthBendix::gilman_digraph.
+      //! \ref gilman_digraph.
       uint64_t size() override;
 
       bool equal_to(std::string const&, std::string const&) override;
@@ -601,8 +609,14 @@ namespace libsemigroups {
       // FpSemigroupInterface - non-pure virtual member functions - public
       //////////////////////////////////////////////////////////////////////////
 
+#ifndef DOXYGEN_SHOULD_SKIP_THIS
+      // The following are required for overload resolution.
+      // Documented in FpSemigroupInterface.
+      // Sphinx/doxygen get confused by this, so we don't allow Doxygen to parse
+      // these two declarations.
       using FpSemigroupInterface::equal_to;
       using FpSemigroupInterface::normal_form;
+#endif
 
      private:
       //////////////////////////////////////////////////////////////////////////
@@ -621,9 +635,7 @@ namespace libsemigroups {
 
       void run_impl() override;
 
-      bool finished_impl() const override {
-        return confluent();
-      }
+      bool finished_impl() const override;
 
       bool is_obviously_infinite_impl() override;
       bool is_obviously_finite_impl() override;
@@ -647,10 +659,10 @@ namespace libsemigroups {
 
       struct Settings {
         Settings();
-        size_t          _check_confluence_interval;
-        size_t          _max_overlap;
-        size_t          _max_rules;
-        policy::overlap _overlap_policy;
+        size_t           _check_confluence_interval;
+        size_t           _max_overlap;
+        size_t           _max_rules;
+        options::overlap _overlap_policy;
       } _settings;
 
       // Forward declarations
@@ -669,18 +681,21 @@ namespace libsemigroups {
     //! monoids.
     //!
     //! This page contains details of the member functions of the class
-    //! libsemigroups::congruence::KnuthBendix.
+    //! congruence::KnuthBendix.
     //!
-    //! \sa libsemigroups::fpsemigroup::KnuthBendix.
+    //! \sa fpsemigroup::KnuthBendix.
+    //!
+    //! \note congruence::KnuthBendix can only be used to compute 2-sided
+    //! congruences.
     //!
     //! \par Example
     //! \code
     //! KnuthBendix kb;
-    //! kb.set_nr_generators(2);
+    //! kb.set_number_of_generators(2);
     //! kb.add_pair({0, 0, 0}, {0});
     //! kb.add_pair({0}, {1, 1});
     //!
-    //! kb.nr_classes();                            // 5
+    //! kb.number_of_classes();                            // 5
     //! kb.word_to_class_index({0, 0, 1});          // 4
     //! kb.word_to_class_index({0, 0, 0, 0, 1});    // 4
     //! kb.word_to_class_index({0, 1, 1, 0, 0, 1}); // 4
@@ -694,16 +709,16 @@ namespace libsemigroups {
       // KnuthBendix - constructors - public
       ////////////////////////////////////////////////////////////////////////////
 
-      //! Constructs a KnuthBendix instance with no generating pairs.
+      //! Default constructor.
       //!
-      //! \par Parameters
+      //! \parameters
       //! (None)
       //!
       //! \complexity
       //! Constant.
       KnuthBendix();
 
-      //! Constructs a KnuthBendix instance from a FroidurePin instance.
+      //! Constructs from FroidurePin instance.
       //!
       //! \param fp the FroidurePin instance.
       //!
@@ -725,8 +740,7 @@ namespace libsemigroups {
                       "FroidurePinBase");
       }
 
-      //! Constructs a congruence::KnuthBendix instance from an
-      //! fpsemigroup::KnuthBendix.
+      //! Construct from fpsemigroup::KnuthBendix.
       //!
       //! A congruence::KnuthBendix instance simply wraps an
       //! fpsemigroup::KnuthBendix, and provides an API compatible with the
@@ -739,7 +753,7 @@ namespace libsemigroups {
       //! rules of \p copy.
       explicit KnuthBendix(fpsemigroup::KnuthBendix const& copy);
 
-      //! Constructs a KnuthBendix instance from a FroidurePin instance.
+      //! Construct from \shared_ptr to FroidurePin instance.
       //!
       //! \param fpb the FroidurePin instance.
       //!
@@ -751,8 +765,7 @@ namespace libsemigroups {
       //! The FroidurePin instance used in construction is not copied.
       explicit KnuthBendix(std::shared_ptr<FroidurePinBase> fpb);
 
-      //! Copy construct a congruence::KnuthBendix instance from a
-      //! congruence::KnuthBendix.
+      //! Copy constructor
       //!
       //! \param copy the congruence::KnuthBendix to be copied.
       //!
@@ -774,7 +787,7 @@ namespace libsemigroups {
 
       //! Returns the underlying fpsemigroup::KnuthBendix.
       //!
-      //! \par Parameters
+      //! \parameters
       //! (None)
       //!
       //! \complexity
@@ -801,7 +814,7 @@ namespace libsemigroups {
       ////////////////////////////////////////////////////////////////////////////
 
       word_type class_index_to_word_impl(class_index_type) override;
-      size_t    nr_classes_impl() override;
+      size_t    number_of_classes_impl() override;
       std::shared_ptr<FroidurePinBase> quotient_impl() override;
       class_index_type word_to_class_index_impl(word_type const&) override;
       void             run_impl() override;
@@ -811,7 +824,7 @@ namespace libsemigroups {
       ////////////////////////////////////////////////////////////////////////////
 
       void add_pair_impl(word_type const&, word_type const&) override;
-      void set_nr_generators_impl(size_t) override;
+      void set_number_of_generators_impl(size_t) override;
       bool is_quotient_obviously_finite_impl() override;
       bool is_quotient_obviously_infinite_impl() override;
 
@@ -822,5 +835,91 @@ namespace libsemigroups {
       std::unique_ptr<fpsemigroup::KnuthBendix> _kb;
     };
   }  // namespace congruence
+
+  namespace presentation {
+
+    //! Return an iterator pointing at the left hand side of a redundant rule.
+    //!
+    //! This function is defined in ``knuth-bendix.hpp``.
+    //!
+    //! Starting with the last rule in the presentation, this function attempts
+    //! to run the Knuth-Bendix algorithm on the rules of the presentation
+    //! except for the given omitted rule. For every such omitted rule,
+    //! Knuth-Bendix is run for the length of time indicated by the second
+    //! parameter \p t, and then it is checked if the omitted rule can be shown
+    //! to be redundant (rewriting both sides of the omitted rule using the
+    //! other rules using the output of the, not necessarily finished,
+    //! Knuth-Bendix algorithm).
+    //!
+    //! If the omitted rule can be shown to be redundant in this way, then an
+    //! iterator pointing to its left hand side is returned.
+    //!
+    //! If no rule can be shown to be redundant in this way, then an iterator
+    //! pointing to `p.cend()` is returned.
+    //!
+    //! \tparam T type of the 2nd parameter (time to try running Knuth-Bendix).
+    //! \param p the presentation
+    //! \param t time to run KnuthBendix for every omitted rule
+    //!
+    //! \warning The progress of the Knuth-Bendix algorithm may differ between
+    //! different calls to this function even if the parameters are identical.
+    //! As such this is non-deterministic, and may produce different results
+    //! with the same input.
+    template <typename T>
+    auto redundant_rule(Presentation<std::string>& p, T t) {
+      p.validate();
+      for (auto omit = p.rules.crbegin(); omit != p.rules.crend(); omit += 2) {
+        fpsemigroup::KnuthBendix kb;
+        kb.set_alphabet(p.alphabet());
+
+        for (auto it = p.rules.crbegin(); it != omit; it += 2) {
+          kb.add_rule(*it, *(it + 1));
+        }
+        for (auto it = omit + 2; it < p.rules.crend(); it += 2) {
+          kb.add_rule(*it, *(it + 1));
+        }
+        kb.run_for(t);
+        if (kb.rewrite(*omit) == kb.rewrite(*(omit + 1))) {
+          return (omit + 1).base() - 1;
+        }
+      }
+      return p.rules.cend();
+    }
+
+    //! Return an iterator pointing at the left hand side of a redundant rule.
+    //!
+    //! This function is defined in ``knuth-bendix.hpp``.
+    //!
+    //! Starting with the last rule in the presentation, this function attempts
+    //! to run the Knuth-Bendix algorithm on the rules of the presentation
+    //! except for the given omitted rule. For every such omitted rule,
+    //! Knuth-Bendix is run for the length of time indicated by the second
+    //! parameter \p t, and then it is checked if the omitted rule can be shown
+    //! to be redundant (rewriting both sides of the omitted rule using the
+    //! other rules using the output of the, not necessarily finished,
+    //! Knuth-Bendix algorithm).
+    //!
+    //! If the omitted rule can be shown to be redundant in this way, then an
+    //! iterator pointing to its left hand side is returned.
+    //!
+    //! If no rule can be shown to be redundant in this way, then an iterator
+    //! pointing to `p.cend()` is returned.
+    //!
+    //! \tparam W type of words in the Presentation
+    //! \tparam T type of the 2nd parameter (time to try running Knuth-Bendix).
+    //! \param p the presentation
+    //! \param t time to run KnuthBendix for every omitted rule
+    //!
+    //! \warning The progress of the Knuth-Bendix algorithm may differ between
+    //! different calls to this function even if the parameters are identical.
+    //! As such this is non-deterministic, and may produce different results
+    //! with the same input.
+    template <typename W, typename T>
+    auto redundant_rule(Presentation<W>& p, T t) {
+      auto pp = make<Presentation<std::string>>(p);
+      return p.rules.cbegin()
+             + std::distance(pp.rules.cbegin(), redundant_rule(pp, t));
+    }
+  }  // namespace presentation
 }  // namespace libsemigroups
 #endif  // LIBSEMIGROUPS_KNUTH_BENDIX_HPP_
